import java.util.PriorityQueue;
import java.util.Scanner;

// Single occurence of the square to be explored in the heap
// Note that we do not remove the elements from the heap,
// but a square might be inserted twice with different times
class SquareInHeap implements Comparable<SquareInHeap> {
	// Location
	public int i;
	public int j;
	// Best time of this exploration
	public int time;
	// Where did we come from?
	// (For path reconstruction. Not needed for length.) 
	public SquareInHeap from;
	
	// Comparator for the heap
	@Override
	public int compareTo(SquareInHeap sq) {
		if (sq.time < time)
			return 1;
		if (sq.time > time)
			return -1;
		return 0;
	}
	
	// Trivial constructor
	public SquareInHeap(int i0, int j0, int time0, SquareInHeap from0) {
		i = i0;
		j = j0;
		time = time0;
		from = from0;
	}
}

/* NB: For the judge to run the program, do not modify the declaration of the class Main or
 *     method main() below. The class has to be declared as "class Main { ... }"
 *     and the method as "public static void main(String[] args) { ... }" */
class Main {

	public static void main(String[] args) {

		// Read from the standard input with java.util.Scanner
		Scanner scanner = new Scanner(System.in);
		
		// Read the number of cases and loop over the cases
		int case_no, cases = scanner.nextInt();
		for (case_no = 0; case_no < cases; case_no++) {
			
			// Read the jungle size
			int n = scanner.nextInt();
			
			// Create a new 2D array for the times
			int[][] T = new int[n][n]; 
			// Smallest time to get here, -1 if unknown
			int[][] time = new int[n][n]; 
			// Start position
			int i0=-1, j0=-1;
			
			// Read the square times, initialize the shortest times to -1, find the start
			int i, j;
			for (i = 0; i < n; i++) {
				for (j = 0; j < n; j++) {
					T[i][j] = scanner.nextInt();
					if (T[i][j] == 0) {
						i0 = i;
						j0 = j;
					}
					time[i][j] = -1;
				}
			}
			
			// Create heap and initialize the computation at the start
			PriorityQueue<SquareInHeap> heap = new PriorityQueue<SquareInHeap>();
			heap.add(new SquareInHeap(i0, j0, 0, null));
			
			// Loop exploring a single square minimal in the heap until
			// we hit the map edge
			while (true) {
				// Pop the smallest (possibly unexplored) vertex from the heap
				// Note the heap of next squares can not run out in this task
				SquareInHeap next = heap.remove();
				
				// If it has been explored, ignore it, otherwise explore
				if (time[next.i][next.j] < 0) {
					
					// Is it an edge square?
					if (next.i == 0 || next.i == n - 1 || next.j == 0 || next.j == n - 1) {
						// We are out! Finish the program and break the "while" loop
						System.out.println(next.time);
						
						// Bonus: write the best path found to "error" output
						// (there may be such paths more of equal length, we show just one)
						System.err.print("Path (reversed): ");
						while (next != null) {
							System.err.print(next.i + "," + next.j + " ");
							next = next.from;
						}
						System.err.println();
						
						break;
					}
					
					// Update time[][] for "next"
					time[next.i][next.j] = next.time;
					
					// And explore the 4 neighbors if unexplored
					// (we are not on the edge, so all exist)
					if (time[next.i - 1][next.j] < 0)
						heap.add(new SquareInHeap(next.i - 1, next.j, next.time + T[next.i - 1][next.j], next));
					if (time[next.i + 1][next.j] < 0)
						heap.add(new SquareInHeap(next.i + 1, next.j, next.time + T[next.i + 1][next.j], next));
					if (time[next.i][next.j - 1] < 0)
						heap.add(new SquareInHeap(next.i, next.j - 1, next.time + T[next.i][next.j - 1], next));
					if (time[next.i][next.j + 1] < 0)
						heap.add(new SquareInHeap(next.i, next.j + 1, next.time + T[next.i][next.j + 1], next));
				}
			}
		}
		scanner.close();
	}
}
